import numpy as np
import pandas as pd
from datetime import datetime
from scipy.stats import itemfreq
from sklearn.model_selection import train_test_split
from sklearn.model_selection import cross_val_score
from sklearn.preprocessing import StandardScaler

from sklearn.tree import DecisionTreeClassifier
from sklearn.ensemble import RandomForestClassifier
from sklearn.neural_network import MLPClassifier
from sklearn.svm import LinearSVC

from keras.models import Model, Sequential
from keras.layers import Input, Dense, Dropout
from keras import regularizers
from keras.optimizers import RMSprop
from keras.utils.training_utils import multi_gpu_model
import tensorflow as tf
import keras

np.random.seed(2018)


def show(message):
    print(message)

ENCODING_DIM = 100
AUTOENCODER_EPOCHS = 100
CLASSIFIER_EPOCHS = 10
PRINT_PERIOD = 5

dataset = np.loadtxt("data/data.csv", delimiter=",")
print(dataset.shape)
X = dataset[:,:-1]
Y = dataset[:,-1]
# print(X, Y)
num_classes = np.size(np.unique(Y))
print(num_classes)
scaler = StandardScaler()
scaled_X = scaler.fit_transform(X)

X_train, X_test, y_train, y_test = train_test_split(scaled_X, Y, test_size=0, random_state=42)
print(X_train.shape, X_test.shape)
NUM_ROWS = len(X_train)
NUM_COLS = len(X_train[0])
print(NUM_ROWS, NUM_COLS)

input_data = Input(shape=(NUM_COLS,))

encoded = Dense(25000, activation='relu')(input_data)
encoded = Dense(10000, activation='relu')(encoded)
encoded = Dense(2000, activation='relu')(encoded)
encoded = Dense(500, activation='relu')(encoded)
encoded = Dense(ENCODING_DIM, activation='relu')(encoded)

decoded = Dense(500, activation='relu')(encoded)
decoded = Dense(2000, activation='relu')(decoded)
decoded = Dense(10000, activation='relu')(decoded)
decoded = Dense(25000, activation='relu')(decoded)
decoded = Dense(NUM_COLS, activation='sigmoid')(decoded)

autoencoder = Model(input_data, decoded)
autoencoder.compile(optimizer='adam', loss='mean_squared_error')

autoencoder.fit(X_train, X_train,
                epochs=AUTOENCODER_EPOCHS,
                shuffle=True,
                validation_data=[X_test,X_test],
                verbose=1,
                )
encoder = Model(input = input_data, output = encoded)

encoded_input = Input(shape = (ENCODING_DIM, ))
encoded_out = encoder.predict(X_train)

np.savetxt("data/encoded.csv", encoded_out, delimiter=",")

