# -*- coding: utf-8 -*-
import numpy as np
import pandas as pd
from scipy.stats import itemfreq
from sklearn.model_selection import train_test_split
from sklearn.model_selection import cross_val_score
from sklearn.preprocessing import StandardScaler

from sklearn.tree import DecisionTreeClassifier
from sklearn.ensemble import RandomForestClassifier
from sklearn.neural_network import MLPClassifier
from sklearn.neighbors import KNeighborsClassifier
from sklearn.naive_bayes import GaussianNB
from sklearn.svm import LinearSVC

from keras.models import Model, Sequential
from keras.layers import Input, Dense, Dropout
from keras import regularizers
from keras.optimizers import RMSprop

np.random.seed(2018)

def show(message):
    print(message)

ENCODING_DIM = 100
AUTOENCODER_EPOCHS = 100
CLASSIFIER_EPOCHS = 10
PRINT_PERIOD = 5

dataset = np.genfromtxt("data/data.csv", delimiter=",", filling_values=0)
print(dataset.shape)
X = dataset[:,:-1]
Y = dataset[:,-1]
# print(X, Y)
num_classes = np.size(np.unique(Y))
print(num_classes)
scaler = StandardScaler()
scaled_X = scaler.fit_transform(X)

X_train, X_test, y_train, y_test = train_test_split(scaled_X, Y, test_size=0, random_state=42)
print(X_train.shape, X_test.shape)
NUM_ROWS = len(X_train)
NUM_COLS = len(X_train[0])
print(NUM_ROWS, NUM_COLS)

encoded_out = np.loadtxt('encoded.csv', delimiter=',')

message = ''
def single_random_train(name, X, Y, clf, encoded_out=encoded_out):    
    X_train, X_test, y_train, y_test = train_test_split(X, Y, test_size=0.25, random_state=42)
    score = cross_val_score(clf, X_train, y_train, cv=5, n_jobs=-1)
    message = name, ": Accuracy: %0.4f (+/- %0.4f)" % (score.mean(), score.std() * 2)
    message = ' '.join([str(x) for x in message])
    show(message)

single_random_train(name='SVM', X=encoded_out, Y=y_train, clf=LinearSVC(random_state=0))
single_random_train(name='DT', X=encoded_out, Y=y_train, clf=DecisionTreeClassifier(criterion='entropy', presort=True, random_state=0))
single_random_train(name='RF', X=encoded_out, Y=y_train, clf=RandomForestClassifier(random_state=0))
single_random_train(name='MLP', X=encoded_out, Y=y_train, clf=MLPClassifier(hidden_layer_sizes=800, random_state=0))
single_random_train(name='KNN', X=encoded_out, Y=y_train, clf=KNeighborsClassifier(3))
single_random_train(name='NB', X=encoded_out, Y=y_train, clf=GaussianNB())
