import numpy as np
import pandas as pd
from scipy.stats import itemfreq
from sklearn.model_selection import train_test_split
from sklearn.model_selection import cross_val_score
from sklearn.preprocessing import StandardScaler
from sklearn.preprocessing import MinMaxScaler
from sklearn.model_selection import GridSearchCV
from sklearn import tree

from sklearn.decomposition import NMF
from sklearn.decomposition import FastICA
from sklearn.decomposition import IncrementalPCA
from sklearn.decomposition import KernelPCA
from sklearn.decomposition import NMF
from sklearn.decomposition import PCA

from sklearn.neural_network import MLPClassifier
from sklearn.svm import LinearSVC
from sklearn.tree import DecisionTreeClassifier
from sklearn.ensemble import RandomForestClassifier
from sklearn.neighbors import KNeighborsClassifier
from sklearn.naive_bayes import GaussianNB

# np.random.seed(2018)

ENCODING_DIM = 100
CLASSIFIER_EPOCHS = 10
PRINT_PERIOD = 10000

dataset = np.loadtxt("data/data.csv", delimiter=",")
print(dataset.shape)
X = dataset[:, :-1]
Y = dataset[:, -1]
# print(X, Y)
num_classes = np.size(np.unique(Y))
print(num_classes)
scaler = MinMaxScaler()
X = scaler.fit_transform(X)

X_train, X_test, y_train, y_test = train_test_split(
    X, Y, test_size=0)
print(X_train.shape, X_test.shape)
NUM_ROWS = len(X_train)
NUM_COLS = len(X_train[0])
print(NUM_ROWS, NUM_COLS)

outs = []


def single_random_train(data_name, X, Y, decomposer, clf):
    X_train, X_test, y_train, y_test = train_test_split(
        X, Y, test_size=0.25)
    score = cross_val_score(clf, X_train, y_train, cv=4, n_jobs=-1)
    out = [data_name, dmap[decomposer], cmap[classifier], score.mean()]
    print(data_name, dmap[decomposer], cmap[classifier],
          ": Accuracy: %0.4f (+/- %0.4f)" % (score.mean(), score.std() * 2))
    outs.append(out)


a = LinearSVC()
b = DecisionTreeClassifier()
c = RandomForestClassifier()
d = MLPClassifier(hidden_layer_sizes=800, max_iter=500)
e = KNeighborsClassifier(3)
f = GaussianNB()
classifiers = [a, b, c, d, e, f]
cmap = {
    a: "SVM",
    b: "Decision Tree",
    c: "Random Forest",
    d: "MLP",
    e: "KNN",
    f: "Naive Bayes"
}

a = FastICA(n_components=10)
b = IncrementalPCA(n_components=10)
c = KernelPCA(n_components=10, n_jobs=-1)
d = NMF(n_components=10)
e = PCA(n_components=10)

decompositions = [a, b, c, d, e]

dmap = {a: "FastICA",
        b: "Incremental PCA",
        c: "Kernel PCA",
        d: "NMF",
        e: "PCA"
        }

datasets = ['data/miRNA.csv', 'data/clean_methylation.csv',
            'data/clean_mRNA.csv', 'data/data.csv']

for filename in datasets:
    print(filename)
    dataset = np.genfromtxt(filename, delimiter=",", filling_values=0)
    X = dataset[:, :-1]
    Y = dataset[:, -1]
    scaler = MinMaxScaler()
    scaled_X = scaler.fit_transform(X)
    for decomposer in decompositions:
        decomposer.fit(scaled_X)
        for classifier in classifiers:
            single_random_train(data_name=filename, X=scaled_X, Y=Y, decomposer=decomposer,
                                clf=classifier)

print(out)
